@@ -29,7 +29,6 @@
 #include <kcolordialog.h>
 #include <kconfig.h>
 #include <kdebug.h>
-#include <kkeydialog.h>
 
 // application specific includes
 #include "kcoloredit.h"
@@ -51,7 +50,8 @@ KColorEditApp::KColorEditApp() : KMainWindow(0) {
   initDocument();
   initView();
 
-  createGUI();
+  resize(606,400);
+  setupGUI();
 
   readOptions();
 
@@ -69,7 +69,6 @@ void KColorEditApp::initActions()
   KStdAction::saveAs( this, SLOT( slotFileSaveAs() ), actionCollection() );
   KStdAction::close( this, SLOT( slotClose() ), actionCollection() );
   KStdAction::quit( this, SLOT( slotQuit() ), actionCollection() );
-  KStdAction::keyBindings( this, SLOT( slotConfigureKeys() ), actionCollection() );
   m_actSave = KStdAction::save( this, SLOT( slotFileSave() ),
           actionCollection() );
   m_actRecent = KStdAction::openRecent( this,
@@ -93,21 +92,13 @@ void KColorEditApp::initActions()
   m_actNames = new KToggleAction( i18n("Show &Color Names"), KShortcut(), this,
           SLOT( slotViewColorNames() ), actionCollection(),
           "color_view_names" );
+  m_actNames->setCheckedState(i18n("Hide &Color Names"));
   m_actPalette = new KAction( i18n("From &Palette"), KShortcut(), this,
           SLOT( slotColorFromPalette() ), actionCollection(),
           "color_from_palette" );
   ( void ) new KAction( i18n("From &Screen"), KShortcut(), this,
           SLOT( slotColorFromScreen() ), actionCollection(),
           "color_from_screen" );
-
-  // Settings Menu
-  createStandardStatusBarAction();
-  setStandardToolBarMenuEnabled(true);
-}
-
-void KColorEditApp::slotConfigureKeys()
-{
-  KKeyDialog::configure( actionCollection(), this );
 }
 
 void KColorEditApp::initStatusBar()
@@ -237,7 +228,7 @@ void KColorEditApp::slotFileOpen() {
           KMessageBox::sorry(0, doc->errorString());
         } else {
           setCaption(doc->title());
-          m_actRecent->addURL( fileToOpen );
+          m_actRecent->addURL( KURL::fromPathOrURL( fileToOpen ) );
         }
       }
     }
@@ -269,8 +260,8 @@ bool KColorEditApp::slotFileSaveAs()
         if(newName.isEmpty())
             result = false;
         else {
-	    QFileInfo saveAsInfo(newName);
-	    if(!saveAsInfo.exists() ||
+      QFileInfo saveAsInfo(newName);
+      if(!saveAsInfo.exists() ||
                KMessageBox::questionYesNo( this,
                        i18n("A Document with this name already exists.\n"
                             "Do you want to overwrite it?"),
@@ -320,7 +311,7 @@ void KColorEditApp::slotQuit()
       // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
       // the window and the application stay open.
       if(!w->close())
-	break;
+  break;
     }
   }
 }
