@@ -4,7 +4,7 @@
 // Methods for dviwin which deal with "\special" commands found in the
 // DVI file
 
-// Copyright 2000--2003, Stefan Kebekus (kebekus@kde.org).
+// Copyright 2000--2004, Stefan Kebekus (kebekus@kde.org).
 
 
 #include <kdebug.h>
@@ -14,6 +14,7 @@
 #include <qdir.h>
 #include <qfile.h>
 #include <qfileinfo.h>
+#include <qimage.h>
 #include <qstringlist.h>
 
 #include "dviwin.h"
@@ -25,7 +26,7 @@
 
 extern QPainter foreGroundPaint;
 
-void dviWindow::printErrorMsgForSpecials(QString msg)
+void dviRenderer::printErrorMsgForSpecials(QString msg)
 {
   if (dviFile->errorCounter < 25) {
     kdError(4300) << msg << endl;
@@ -130,7 +131,7 @@ QColor parseColorSpecification(QString colorSpec)
 
 
 
-void dviWindow::color_special(QString cp)
+void dviRenderer::color_special(QString cp)
 {
   cp = cp.stripWhiteSpace();
   
@@ -169,7 +170,7 @@ void dviWindow::color_special(QString cp)
 }
 
 
-void dviWindow::html_href_special(QString cp)
+void dviRenderer::html_href_special(QString cp)
 {
   cp.truncate(cp.find('"'));
   
@@ -180,7 +181,7 @@ void dviWindow::html_href_special(QString cp)
 }
 
 
-void dviWindow::html_anchor_end(void)
+void dviRenderer::html_anchor_end(void)
 {
 #ifdef DEBUG_SPECIAL
   kdDebug(4300) << "HTML-special, anchor-end" << endl;
@@ -193,7 +194,7 @@ void dviWindow::html_anchor_end(void)
 }
 
 
-void dviWindow::source_special(QString cp)
+void dviRenderer::source_special(QString cp)
 {
   // only when rendering really takes place: set source_href to the
   // current special string. When characters are rendered, the
@@ -224,7 +225,8 @@ void parse_special_argument(QString strg, const char *argument_name, int *variab
   }
 }
 
-void dviWindow::epsf_special(QString cp)
+
+void dviRenderer::epsf_special(QString cp)
 {
 #ifdef DEBUG_SPECIAL
   kdDebug(4300) << "epsf-special: psfile=" << cp <<endl;
@@ -269,6 +271,43 @@ void dviWindow::epsf_special(QString cp)
   parse_special_argument(include_command, "rhi=", &rhi);
   parse_special_argument(include_command, "angle=", &angle);
 
+
+  // If the file name ends in 'png', 'gif', 'jpg' or 'jpeg', we assume
+  // that this is NOT a PostScript file, and we need to draw that
+  // here.
+  QString ending = EPSfilename.section('.', -1).lower();
+  bool isGFX = false;
+  if ((ending == "png") || (ending == "gif") || (ending == "jpg") || (ending == "jpeg") || (ending == "mng"))
+    isGFX = true;
+
+  // So, if we do not have a PostScript file, but a graphics file, and
+  // if that file exists, we draw it here.
+  if (isGFX && QFile::exists(EPSfilename)) {
+    // Don't show PostScript, just draw the bounding box. For this,
+    // calculate the size of the bounding box in Pixels. 
+    double bbox_width  = urx - llx;
+    double bbox_height = ury - lly;
+    
+    if ((rwi != 0)&&(bbox_width != 0)) {
+      bbox_height *= rwi/bbox_width;
+      bbox_width  = rwi;
+    }
+    if ((rhi != 0)&&(bbox_height != 0)) {
+      bbox_width  *= rhi/bbox_height;
+      bbox_height = rhi;
+    }
+
+    double fontPixelPerDVIunit = dviFile->getCmPerDVIunit() * 1200.0/2.54;
+    
+    bbox_width  *= 0.1 * 65536.0*fontPixelPerDVIunit / shrinkfactor;
+    bbox_height *= 0.1 * 65536.0*fontPixelPerDVIunit / shrinkfactor;
+    
+    QImage image(EPSfilename);
+    image = image.smoothScale(bbox_width, bbox_height);
+    foreGroundPaint.drawImage( ((int) ((currinf.data.dvi_h) / (shrinkfactor * 65536))), currinf.data.pxl_v - (int)bbox_height, image);
+    return;
+  }
+
   if (!_postscript || !QFile::exists(EPSfilename)) {
     // Don't show PostScript, just draw the bounding box. For this,
     // calculate the size of the bounding box in Pixels. 
@@ -284,8 +323,7 @@ void dviWindow::epsf_special(QString cp)
       bbox_height = rhi;
     }
 
-    double fontPixelPerDVIunit = dviFile->getCmPerDVIunit() * 
-      MFResolutions[font_pool.getMetafontMode()]/2.54;
+    double fontPixelPerDVIunit = dviFile->getCmPerDVIunit() * 1200.0/2.54;
     
     bbox_width  *= 0.1 * 65536.0*fontPixelPerDVIunit / shrinkfactor;
     bbox_height *= 0.1 * 65536.0*fontPixelPerDVIunit / shrinkfactor;
@@ -311,7 +349,7 @@ void dviWindow::epsf_special(QString cp)
 }
 
 
-void dviWindow::TPIC_flushPath_special(void)
+void dviRenderer::TPIC_flushPath_special(void)
 {
 #ifdef DEBUG_SPECIAL
   kdDebug(4300) << "TPIC special flushPath" << endl;
@@ -322,14 +360,14 @@ void dviWindow::TPIC_flushPath_special(void)
     return;
   }
 
-  QPen pen(Qt::black, (int)(penWidth_in_mInch*xres*_zoom/1000.0 + 0.5));  // Sets the pen size in milli-inches
+  QPen pen(Qt::black, (int)(penWidth_in_mInch*resolutionInDPI/1000.0 + 0.5));  // Sets the pen size in milli-inches
   foreGroundPaint.setPen(pen);
   foreGroundPaint.drawPolyline(TPIC_path, 0, number_of_elements_in_path);
   number_of_elements_in_path = 0;
 }
 
 
-void dviWindow::TPIC_addPath_special(QString cp)
+void dviRenderer::TPIC_addPath_special(QString cp)
 {
 #ifdef DEBUG_SPECIAL
   kdDebug(4300) << "TPIC special addPath: " << cp << endl;
@@ -349,8 +387,8 @@ void dviWindow::TPIC_addPath_special(QString cp)
     return;
   }
   
-  int x = (int)( currinf.data.dvi_h/(shrinkfactor*65536.0) + xKoord*xres*_zoom/1000.0 + 0.5 );
-  int y = (int)( currinf.data.pxl_v + yKoord*xres*_zoom/1000.0 + 0.5 );
+  int x = (int)( currinf.data.dvi_h/(shrinkfactor*65536.0) + xKoord*resolutionInDPI/1000.0 + 0.5 );
+  int y = (int)( currinf.data.pxl_v + yKoord*resolutionInDPI/1000.0 + 0.5 );
   
   // Initialize the point array used to store the path
   if (TPIC_path.size() == 0) 
@@ -361,7 +399,7 @@ void dviWindow::TPIC_addPath_special(QString cp)
 }
 
 
-void dviWindow::TPIC_setPen_special(QString cp)
+void dviRenderer::TPIC_setPen_special(QString cp)
 {
 #ifdef DEBUG_SPECIAL
   kdDebug(4300) << "TPIC special setPen: " << cp << endl;
@@ -378,7 +416,7 @@ void dviWindow::TPIC_setPen_special(QString cp)
 }
 
 
-void dviWindow::applicationDoSpecial(char *cp)
+void dviRenderer::applicationDoSpecial(char *cp)
 {
   QString special_command(cp);
 
@@ -450,7 +488,7 @@ void dviWindow::applicationDoSpecial(char *cp)
     // chracter drawing routines will now underline all characters in
     // blue to point out that they correspond to a hyperlink. Also, as
     // soon as characters are drawn, the drawing routines will
-    // allocate a DVI_Hyperlink and add it to the top of the vector
+    // allocate a Hyperlink and add it to the top of the vector
     // currentlyDrawnPage->hyperLinkList.
     if (special_command == "ps:SDict begin H.S end") {
       // At this stage, the vector 'hyperLinkList' should not contain
@@ -520,7 +558,7 @@ void dviWindow::applicationDoSpecial(char *cp)
     if (special_command.startsWith("ps:SDict begin [") && special_command.endsWith(" pdfmark end")) {
       if (!currentlyDrawnPage->hyperLinkList.isEmpty()) {
 	QString targetName = special_command.section('(', 1, 1).section(')', 0, 0);
-	QValueVector<DVI_Hyperlink>::iterator it;
+	QValueVector<Hyperlink>::iterator it;
         for( it = currentlyDrawnPage->hyperLinkList.begin(); it != currentlyDrawnPage->hyperLinkList.end(); ++it ) 
 	  if (it->linkText == "glopglyph")
 	    it->linkText = targetName;
@@ -529,6 +567,33 @@ void dviWindow::applicationDoSpecial(char *cp)
     }
   }
   
+  // Detect text rotation specials that are included by the graphicx
+  // package. If one of these specials is found, the state of the
+  // painter is saved, and the coordinate system is rotated
+  // accordingly
+  if (special_command.startsWith("ps: gsave currentpoint currentpoint translate ") && 
+      special_command.endsWith(" neg rotate neg exch neg exch translate") ) {
+    bool ok;
+    double angle = special_command.section(' ', 5, 5).toDouble(&ok);
+    if (ok == true) {
+      int x = ((int) ((currinf.data.dvi_h) / (shrinkfactor * 65536)));
+      int y = currinf.data.pxl_v;
+      
+      foreGroundPaint.save();
+      // Rotate about the current point
+      foreGroundPaint.translate(x,y);
+      foreGroundPaint.rotate(-angle);
+      foreGroundPaint.translate(-x,-y);
+    } else
+      printErrorMsgForSpecials( i18n("Error in DVIfile '%1', page %2. Could not interpret angle in text rotation special." ).
+				arg(dviFile->filename).arg(current_page));
+  }
+
+  // The graphicx package marks the end of rotated text with this
+  // special. The state of the painter is restored.
+  if (special_command == "ps: currentpoint grestore moveto") {
+    foreGroundPaint.restore();
+  }
   
   // The following special commands are not used here; they are of
   // interest only during the prescan phase. We recognize them here
