@@ -15,7 +15,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- *  $Id: kfile_png.cpp,v 1.22 2003/11/09 15:52:16 orlovich Exp $
+ *  $Id: kfile_png.cpp,v 1.26 2004/12/15 08:20:28 bhards Exp $
  */
 
 #include <stdlib.h>
@@ -32,6 +32,7 @@
 #include <qdatetime.h>
 #include <qdict.h>
 #include <qvalidator.h>
+#include <zlib.h>
 
 // some defines to make it easier
 // don't tell me anything about preprocessor usage :)
@@ -109,31 +110,38 @@ KPngPlugin::KPngPlugin(QObject *parent, const char *name,
     setHint( item, KFileMimeTypeInfo::Size );
     setUnit(item, KFileMimeTypeInfo::Pixels);
 
-    item = addItemInfo(group, "BitDepth", i18n("Bit depth"), QVariant::Int);
+    item = addItemInfo(group, "BitDepth", i18n("Bit Depth"), QVariant::Int);
     setUnit(item, KFileMimeTypeInfo::BitsPerPixel);
 
-    addItemInfo(group, "ColorMode", i18n("Color mode"), QVariant::String);
+    addItemInfo(group, "ColorMode", i18n("Color Mode"), QVariant::String);
     addItemInfo(group, "Compression", i18n("Compression"), QVariant::String);
-    addItemInfo(group, "InterlaceMode", i18n("Interlace mode"),QVariant::String);
+    addItemInfo(group, "InterlaceMode", i18n("Interlace Mode"),QVariant::String);
 }
 
 bool KPngPlugin::readInfo( KFileMetaInfo& info, uint what)
 {
+    if ( info.path().isEmpty() ) // remote file
+        return false;
     QFile f(info.path());
-    f.open(IO_ReadOnly);
+    if ( !f.open(IO_ReadOnly) )
+        return false;
 
-    if (f.size() < 26) return false;
-    // the technical group will be read from the first 26 bytes. If the file
+    QIODevice::Offset fileSize = f.size();
+
+    if (fileSize < 29) return false;
+    // the technical group will be read from the first 29 bytes. If the file
     // is smaller, we can't even read this.
 
     bool readComments = false;
     if (what & (KFileMetaInfo::Fastest |
                 KFileMetaInfo::DontCare |
                 KFileMetaInfo::ContentInfo)) readComments = true;
+    else
+	fileSize = 29; // No need to read more
 
-    uchar *data = new uchar[f.size()+1];
-    f.readBlock(reinterpret_cast<char*>(data), f.size());
-    data[f.size()]='\n';
+    uchar *data = new uchar[fileSize+1];
+    f.readBlock(reinterpret_cast<char*>(data), fileSize);
+    data[fileSize]='\n';
 
     // find the start
     if (data[0] == 137 && data[1] == 80 && data[2] == 78 && data[3] == 71 &&
@@ -190,10 +198,12 @@ bool KPngPlugin::readInfo( KFileMetaInfo& info, uint what)
             index += CHUNK_SIZE(data, index) + CHUNK_HEADER_SIZE;
             KFileMetaInfoGroup commentGroup = appendGroup(info, "Comment");
 
-            while(index<f.size()-12)
+            while(index<fileSize-12)
             {
-                while (index < f.size() - 12 &&
-                       strncmp((char*)CHUNK_TYPE(data,index), "tEXt", 4))
+                while (index < fileSize - 12 &&
+                       strncmp((char*)CHUNK_TYPE(data,index), "tEXt", 4) &&
+                       strncmp((char*)CHUNK_TYPE(data,index), "zTXt", 4))
+
                 {
                     if (!strncmp((char*)CHUNK_TYPE(data,index), "IEND", 4))
                         goto end;
@@ -201,10 +211,10 @@ bool KPngPlugin::readInfo( KFileMetaInfo& info, uint what)
                     index += CHUNK_SIZE(data, index) + CHUNK_HEADER_SIZE;
                 }
 
-                if (index < f.size() - 12)
+                if (index < fileSize - 12)
                 {
-                    // we found a tEXt field
-                    kdDebug(7034) << "We found a tEXt field\n";
+                    // we found a tEXt or zTXt field
+
                     // get the key, it's a null terminated string at the
                     //  chunk start
 
@@ -214,32 +224,81 @@ bool KPngPlugin::readInfo( KFileMetaInfo& info, uint what)
                     for (;key[keysize]!=0; keysize++)
                         // look if we reached the end of the file
                         // (it might be corrupted)
-                        if (8+index+keysize>=f.size())
+                        if (8+index+keysize>=fileSize)
                             goto end;
 
-                    // the text comes after the key, but isn't null terminated
-                    uchar* text = &CHUNK_DATA(data,index, keysize+1);
-                    uint textsize = CHUNK_SIZE(data, index)-keysize-1;
+		    QByteArray arr;
+		    if(!strncmp((char*)CHUNK_TYPE(data,index), "zTXt", 4)) {
+			kdDebug(7034) << "We found a zTXt field\n";
+			// we get the compression method after the key
+			uchar* compressionMethod = &CHUNK_DATA(data,index,keysize+1);
+			if ( *compressionMethod != 0x00 ) {
+			    // then it isn't zlib compressed and we are sunk
+			    kdDebug(7034) << "Non-standard compression method." << endl;
+			    goto end;
+			}
+			// compressed string after the compression technique spec
+			uchar* compressedText = &CHUNK_DATA(data, index, keysize+2);
+			uint compressedTextSize = CHUNK_SIZE(data, index)-keysize-2;
 
-                    // security check, also considering overflow wraparound from the addition --
-                    // we may endup with a /smaller/ index if we wrap all the way around
-                    uint firstIndex       = (uint)(text - data);
-                    uint onePastLastIndex = firstIndex + textsize;
+			// security check, also considering overflow wraparound from the addition --
+			// we may endup with a /smaller/ index if we wrap all the way around
+			uint firstIndex       = (uint)(compressedText - data);
+			uint onePastLastIndex = firstIndex + compressedTextSize;
+			if ( onePastLastIndex > fileSize || onePastLastIndex <= firstIndex)
+			    goto end;
 
-                    if ( onePastLastIndex > f.size() || onePastLastIndex <= firstIndex)
-                        goto end;
+			int uncompressedLen = compressedTextSize * 2; // just a starting point
+			int zlibResult;
+			do {
+			    arr.resize(uncompressedLen);
+			    zlibResult = uncompress((Bytef*)arr.data(), (uLongf*)(&uncompressedLen),
+						    compressedText, compressedTextSize);
+			    if (Z_OK == zlibResult) {
+				// then it is all OK
+				arr.resize(uncompressedLen);
+			    } else if (Z_BUF_ERROR == zlibResult) {
+				// the uncompressedArray needs to be larger
+				// kdDebug(7034) << "doubling size for decompression" << endl;
+				uncompressedLen *= 2;
+			    } else {
+				// something bad happened
+				goto end;
+			    }
+			} while (Z_BUF_ERROR == zlibResult);
 
-                    QByteArray arr(textsize);
-                    arr = QByteArray(textsize).duplicate((const char*)text,
-                                                         textsize);
-                    appendItem(commentGroup,
-                               QString(reinterpret_cast<char*>(key)),
-                               QString(arr));
+			if (Z_OK != zlibResult)
+			    goto end;
+		    } else if (!strncmp((char*)CHUNK_TYPE(data,index), "tEXt", 4)) {
+			kdDebug(7034) << "We found a tEXt field\n";
+			// the text comes after the key, but isn't null terminated
+			uchar* text = &CHUNK_DATA(data,index, keysize+1);
+			uint textsize = CHUNK_SIZE(data, index)-keysize-1;
 
-                    kdDebug(7034) << "adding " << key << " / "
-                                  << QString(arr) << endl;
+			// security check, also considering overflow wraparound from the addition --
+			// we may endup with a /smaller/ index if we wrap all the way around
+			uint firstIndex       = (uint)(text - data);
+			uint onePastLastIndex = firstIndex + textsize;
 
-                    index += CHUNK_SIZE(data, index) + CHUNK_HEADER_SIZE;
+			if ( onePastLastIndex > fileSize || onePastLastIndex <= firstIndex)
+			    goto end;
+
+			arr.resize(textsize);
+			arr = QByteArray(textsize).duplicate((const char*)text,
+							     textsize);
+			
+		    } else {
+			kdDebug(7034) << "We found a field, not expected though\n";
+			goto end;
+		    }
+		    appendItem(commentGroup,
+			       QString(reinterpret_cast<char*>(key)),
+			       QString(arr));
+		    
+		    kdDebug(7034) << "adding " << key << " / "
+				  << QString(arr) << endl;
+
+		    index += CHUNK_SIZE(data, index) + CHUNK_HEADER_SIZE;
                 }
             }
         }
