@@ -15,7 +15,7 @@
   the copyright notice and this notice be preserved on all copies.
 */
 
-/* $Id: dscparse.h,v 1.7 2003/01/14 10:45:15 wgreven Exp $ */
+/* $Id: dscparse.h,v 1.8 2004/10/15 22:06:11 luis_pedro Exp $ */
 
 /* dscparse.h */
 /* Interface for the DSC parser. */
@@ -239,8 +239,7 @@ typedef struct CDSCMEDIA_S {
     CDSCBBOX *mediabox;	/* Used by GSview for PDF MediaBox */
 } CDSCMEDIA;
 
-#define CDSC_KNOWN_MEDIA 11
-extern const CDSCMEDIA dsc_known_media[CDSC_KNOWN_MEDIA];
+extern const CDSCMEDIA dsc_known_media[];
 
 typedef struct CDSCCTM_S { /* used for %%ViewingOrientation */
     float xx;
