@@ -149,7 +149,7 @@ namespace KMrml
         {
             assert( !m_tagName.isEmpty() );
 
-            clear();
+            QValueList<t>::clear();
 
             QDomNodeList list = elem.elementsByTagName( m_tagName );
             for ( uint i = 0; i < list.length(); i++ )
@@ -163,8 +163,8 @@ namespace KMrml
 
         t findByName( const QString& name ) const
         {
-            QValueListConstIterator<t> it = begin();
-            for ( ; it != end(); ++it )
+            QValueListConstIterator<t> it = QValueList<t>::begin();
+            for ( ; it != QValueList<t>::end(); ++it )
             {
                 if ( (*it).name() == name )
                     return *it;
@@ -175,8 +175,8 @@ namespace KMrml
 
         t findById( const QString& id ) const
         {
-            QValueListConstIterator<t> it = begin();
-            for ( ; it != end(); ++it )
+            QValueListConstIterator<t> it = QValueList<t>::begin();
+            for ( ; it != QValueList<t>::end(); ++it )
             {
                 if ( (*it).id() == id )
                     return *it;
@@ -187,8 +187,8 @@ namespace KMrml
 
         QStringList itemNames() const {
             QStringList list;
-            QValueListConstIterator<t> it = begin();
-            for ( ; it != end(); ++it )
+            QValueListConstIterator<t> it = QValueList<t>::begin();
+            for ( ; it != QValueList<t>::end(); ++it )
                 list.append( (*it).name() );
 
             return list;
