@@ -5,7 +5,7 @@
     copyright            : (C) 1999 by Klaas Freitag
     email                : freitag@suse.de
 
-    $Id: kookaview.cpp,v 1.69 2003/10/21 23:10:41 maragato Exp $
+    $Id: kookaview.cpp,v 1.75 2004/11/20 17:27:51 deller Exp $
  ***************************************************************************/
 
 /***************************************************************************
@@ -29,6 +29,7 @@
 #include "kookaview.h"
 #include "resource.h"
 #include "kscandevice.h"
+#include "imgscaninfo.h"
 #include "devselector.h"
 #include "ksaneocr.h"
 #include "img_saver.h"
@@ -119,16 +120,21 @@ KookaView::KookaView( KParts::DockMainWindow *parent, const QCString& deviceToUs
        ctxtmenu->insertTitle(i18n("Image View"));
    m_mainDock->setWidget( img_canvas );
 
+   /** Thumbview **/
    m_dockThumbs = parent->createDockWidget( "Thumbs",
 					    loader->loadIcon( "thumbnail", KIcon::Small ),
 					    0L,  i18n("Thumbnails"));
    m_dockThumbs->setDockSite(KDockWidget::DockFullSite );
 
    /* thumbnail viewer widget */
-   m_thumbview = new ThumbView( m_mainDock );
+   m_thumbview = new ThumbView( m_dockThumbs);
    m_dockThumbs->setWidget( m_thumbview );
 
-   /* make the main dock widget */
+   m_dockThumbs->manualDock( m_mainDock,              // dock target
+			     KDockWidget::DockBottom, // dock site
+			     20 );                  // relation target/this (in percent)
+
+   /** Packager Dock **/
    /* A new packager to contain the already scanned images */
    m_dockPackager = parent->createDockWidget( "Scanpackager",
 					    loader->loadIcon( "palette_color", KIcon::Small ),
@@ -140,7 +146,6 @@ KookaView::KookaView( KParts::DockMainWindow *parent, const QCString& deviceToUs
                          KDockWidget::DockLeft, // dock site
                          30 );                  // relation target/this (in percent)
 
-   m_dockThumbs->manualDock( m_mainDock, KDockWidget::DockBottom, 5 );
 
    connect( packager, SIGNAL(showThumbnails( KFileTreeViewItem* )),
 	    this, SLOT( slShowThumbnails( KFileTreeViewItem* )));
@@ -151,12 +156,11 @@ KookaView::KookaView( KParts::DockMainWindow *parent, const QCString& deviceToUs
     * Create a Kombobox that holds the last folders visible even on the preview page
     */
    m_dockRecent  = parent->createDockWidget( "Recent",
-				loader->loadIcon( "image", KIcon::Small ),
-				0L, i18n("Gallery Folders"));
+					     loader->loadIcon( "image", KIcon::Small ),
+					     0L, i18n("Gallery Folders"));
 
    m_dockRecent->setDockSite(KDockWidget::DockFullSite);
 
-
    QHBox *recentBox = new QHBox( m_dockRecent );
    recentBox->setMargin(KDialog::marginHint());
    QLabel *lab = new QLabel( i18n("Gallery:"), recentBox );
@@ -191,8 +195,8 @@ KookaView::KookaView( KParts::DockMainWindow *parent, const QCString& deviceToUs
    Q_CHECK_PTR(sane);
 
    m_dockScanParam->manualDock( m_dockRecent,              // dock target
-   KDockWidget::DockBottom, // dock site
-   20 );                  // relation target/this (in percent)
+				KDockWidget::DockBottom, // dock site
+				20 );                  // relation target/this (in percent)
    m_dockScanParam->hide();
 
    /* select the scan device, either user or from config, this creates and assembles
@@ -205,7 +209,7 @@ KookaView::KookaView( KParts::DockMainWindow *parent, const QCString& deviceToUs
 
    preview_canvas = new Previewer( m_dockPreview );
    {
-      preview_canvas->setMinimumSize( 100,100);
+       preview_canvas->setMinimumSize( 100,100);
 
       /* since the scan_params will be created in slSelectDevice, do the
        * connections later
@@ -242,9 +246,9 @@ KookaView::KookaView( KParts::DockMainWindow *parent, const QCString& deviceToUs
    }
 
    /* New image created after scanning */
-   connect(sane, SIGNAL(sigNewImage(QImage*)), this, SLOT(slNewImageScanned(QImage*)));
+   connect(sane, SIGNAL(sigNewImage(QImage*,ImgScanInfo*)), this, SLOT(slNewImageScanned(QImage*,ImgScanInfo*)));
    /* New preview image */
-   connect(sane, SIGNAL(sigNewPreview(QImage*)), this, SLOT( slNewPreview(QImage*)));
+   connect(sane, SIGNAL(sigNewPreview(QImage*,ImgScanInfo *)), this, SLOT( slNewPreview(QImage*,ImgScanInfo *)));
 
    connect( sane, SIGNAL( sigScanStart() ), this, SLOT( slScanStart()));
    connect( sane, SIGNAL( sigScanFinished(KScanStat)), this, SLOT(slScanFinished(KScanStat)));
@@ -339,8 +343,6 @@ bool KookaView::slSelectDevice( const QCString& useDevice )
       /* This connects to the selected scanner */
       scan_params = new ScanParams( m_dockScanParam );
       Q_CHECK_PTR(scan_params);
-      // m_dockScanParam->setWidget( scan_params );
-      // m_dockScanParam->show();
 
       if( sane->openDevice( selDevice ) == KSCAN_OK )
       {
@@ -367,7 +369,7 @@ bool KookaView::slSelectDevice( const QCString& useDevice )
 	    /* load the preview image */
 	    if( preview_canvas )
 	    {
-	       preview_canvas->loadPreviewImage( selDevice );
+	       preview_canvas->setPreviewImage( sane->loadPreviewImage() );
 
                /* Call this after the devic is actually open */
                preview_canvas->slConnectScanner( sane );
@@ -477,26 +479,14 @@ void KookaView::print()
     }
 }
 
-void KookaView::slNewPreview( QImage *new_img )
+void KookaView::slNewPreview( QImage *new_img, ImgScanInfo * )
 {
    if( new_img )
    {
       if( ! new_img->isNull() )
       {
-	 ImgSaveStat is_stat = ISS_OK;
-	 ImgSaver img_saver( m_mainDock );
-
-	 is_stat = img_saver.savePreview( new_img, sane->shortScannerName() );
-
-	 if( is_stat != ISS_OK )
-	 {
-	    kdDebug(28000) << "ERROR in saving preview !" << endl;
-	 }
-	 else
-	 {
-	    /* flip preview to front */
-	    m_dockPreview->makeDockVisible();
-	 }
+	 /* flip preview to front */
+	 m_dockPreview->makeDockVisible();
       }
       preview_canvas->newImage( new_img );
    }
@@ -571,6 +561,9 @@ void KookaView::startOCR( KookaImage *img )
           connect( ocrFabric, SIGNAL( newOCRResultText( const QString& )),
                    m_ocrResEdit, SLOT(setText( const QString& )));
 
+	  connect( ocrFabric, SIGNAL( newOCRResultText( const QString& )),
+		   m_dockOCRText, SLOT( show() ));
+	  
           connect( ocrFabric, SIGNAL( repaintOCRResImage( )),
                    img_canvas, SLOT(repaint()));
 
@@ -652,15 +645,10 @@ void KookaView::slAcquireStart( )
    }
 }
 
-void KookaView::slNewImageScanned( QImage* img )
+void KookaView::slNewImageScanned( QImage* img, ImgScanInfo* si )
 {
     KookaImageMeta *meta = new KookaImageMeta;
-
-    KScanOption res ( SANE_NAME_SCAN_RESOLUTION );
-    int resX;
-    res.get(&resX);
-    meta->setScanResolution(resX);
-
+    meta->setScanResolution(si->getXResolution(), si->getYResolution());
     packager->slAddImage(img, meta);
 }
 
@@ -730,8 +718,7 @@ void KookaView::slRotateImage(int angle)
 	    break;
 	 case 180:
 	    emit( signalChangeStatusbar( i18n("Rotate image 180 degrees" )));
-	    resImg = rotateRight( img );
-	    resImg = rotateRight( &resImg );
+	    resImg = rotate180( img );
 	    break;
 	 case 270:
 	 case -90:
@@ -918,7 +905,7 @@ void KookaView::slShowThumbnails(KFileTreeViewItem *dirKfi, bool forceRedraw )
  */
 void KookaView::slStartLoading( const KURL& url )
 {
-   emit( signalChangeStatusbar( i18n("Loading " ) + url.prettyURL()));
+   emit( signalChangeStatusbar( i18n("Loading %1" ).arg( url.prettyURL() ) ));
 
    // if( m_stack->visibleWidget() != img_canvas )
    // {
@@ -977,51 +964,13 @@ void KookaView::slOpenCurrInGraphApp( void )
 QImage KookaView::rotateLeft( QImage *m_img )
 {
    QImage rot;
-
+   
    if( m_img )
    {
-      if ( m_img->depth() == 8)
-      {
-	 rot.create( m_img->height(), m_img->width(),
-		     m_img->depth(), m_img->numColors() );
-	 for (int i=0; i<m_img->numColors(); i++)
-	 {
-	    rot.setColor( i, m_img->color(i) );
-	 }
+       QWMatrix m;
 
-	 unsigned char **ssl = m_img->jumpTable();
-
-	 for (int y=0; y<m_img->height(); y++)
-	 {
-	    unsigned char *p = *ssl++;
-	    unsigned char **dsl = rot.jumpTable();
-	    dsl += m_img->width()-1;
-
-	    for (int x=0; x<m_img->width(); x++)
-	    {
-	       *((*dsl--) + y ) = *p++;
-	    }
-	 }
-      }
-      else
-      {
-	 rot.create( m_img->height(), m_img->width(),
-		     m_img->depth() );
-
-	 QRgb **ssl = (QRgb **)m_img->jumpTable();
-
-	 for (int y=0; y<m_img->height(); y++)
-	 {
-	    QRgb *p = *ssl++;
-	    QRgb **dsl = (QRgb **)rot.jumpTable();
-	    dsl += m_img->width()-1;
-
-	    for (int x=0; x<m_img->width(); x++)
-	    {
-	       *((*dsl--) + y ) = *p++;
-	    }
-	 }
-      }
+       m.rotate(-90);
+       rot = m_img->xForm(m);
    }
    return( rot );
 }
@@ -1029,53 +978,33 @@ QImage KookaView::rotateLeft( QImage *m_img )
 QImage KookaView::rotateRight( QImage *m_img )
 {
    QImage rot;
-
-   if (m_img )
+   
+   if( m_img )
    {
-      if (m_img->depth() == 8)
-      {
-	 rot.create( m_img->height(), m_img->width(),
-		     m_img->depth(), m_img->numColors() );
-	 for (int i=0; i<m_img->numColors(); i++)
-	 {
-	    rot.setColor( i, m_img->color(i) );
-	 }
+       QWMatrix m;
 
-	 unsigned char **ssl = m_img->jumpTable();
-
-	 for (int y=0; y<m_img->height(); y++)
-	 {
-	    unsigned char *p = *ssl++;
-	    unsigned char **dsl = rot.jumpTable();
-
-	    for (int x=0; x<m_img->width(); x++)
-	    {
-	       *((*dsl++) + m_img->height() - y - 1 ) = *p++;
-	    }
-	 }
-      }
-      else
-      {
-	 rot.create( m_img->height(), m_img->width(),
-		     m_img->depth() );
-
-	 QRgb **ssl = (QRgb **)m_img->jumpTable();
-
-	 for (int y=0; y<m_img->height(); y++)
-	 {
-	    QRgb *p = *ssl++;
-	    QRgb **dsl = (QRgb **)rot.jumpTable();
-
-	    for (int x=0; x<m_img->width(); x++)
-	    {
-	       *((*dsl++) + m_img->height() - y - 1 ) = *p++;
-	    }
-	 }
-      }
+       m.rotate(+90);
+       rot = m_img->xForm(m);
    }
    return( rot );
 }
 
+QImage KookaView::rotate180( QImage *m_img )
+{
+   QImage rot;
+   
+   if( m_img )
+   {
+       QWMatrix m;
+
+       m.rotate(+180);
+       rot = m_img->xForm(m);
+   }
+   return( rot );
+}
+
+
+
 void KookaView::connectViewerAction( KAction *action )
 {
    QPopupMenu *popup = img_canvas->contextMenu();
