@@ -80,7 +80,7 @@ public slots:
    void openURL( const KURL& );
    void slotOpenRecent( const KURL& );
    void slotOpenRecentTimer( );
-   
+
    void slotFileNew( );
    void slotFileOpen( );
    void slotFileSave( );
@@ -89,7 +89,7 @@ public slots:
    void slotFilePrint( );
    void slotFileNewWindow( );
    void slotFileClose( );
-   
+
    void slotShowToolbar( );
    void slotShowStatusbar( );
    void slotShowPath( );
@@ -108,10 +108,11 @@ public slots:
    void slotNewCameraView( );
    void slotNewTreeView( );
    void slotNewDialogView( );
-   
+   void slotNewLibraryBrowserView( );
+
    void saveOptions( );
    void restoreOptions( );
-   
+
    void setCaption( const QString& caption = QString::null );
    void statusMsg( const QString& text = QString::null );
    void slotControlPointMsg( const QString& msg = QString::null );
@@ -122,9 +123,10 @@ public slots:
    void slotSelectedLayout( int id );
    void slotSaveViewLayout( );
    void slotViewsMenuAboutToShow( );
-   
+
 protected:
    virtual bool queryClose( );
+	virtual void showEvent( QShowEvent* );
 
 public:
    /**
@@ -137,7 +139,7 @@ public:
    //PMDockWidget* createTreeView( );
    //PMDockWidget* createDialogView( );
    //PMDockWidget* create3DView( PMGLView::PMViewType );
-   
+
 private:
    KRecentFilesAction* m_pRecent;
    KToggleAction* m_pToolbarAction;
@@ -153,6 +155,7 @@ private:
    KAction* m_pNewFrontViewAction;
    KAction* m_pNewBackViewAction;
    KAction* m_pNewCameraViewAction;
+   KAction* m_pNewLibraryBrowserAction;
 
    KActionMenu* m_pViewLayoutsAction;
    KAction* m_pSaveViewLayoutAction;
