@@ -16,17 +16,17 @@
 class RegionGrabber;
 class KSnapshotWidget;
 
-class KSnapshotThumb : public QLabel
+class KSnapshotPreview : public QLabel
 {
     Q_OBJECT
 
     public:
-        KSnapshotThumb(QWidget *parent, const char *name = 0)
+        KSnapshotPreview(QWidget *parent, const char *name = 0)
             : QLabel(parent, name)
         {
             setAlignment(AlignHCenter | AlignVCenter);
         }
-        virtual ~KSnapshotThumb() {}
+        virtual ~KSnapshotPreview() {}
 
     signals:
         void startDrag();
@@ -60,7 +60,7 @@ class KSnapshot : public KDialogBase, virtual public KSnapshotIface
   Q_OBJECT
 
 public:
-  KSnapshot(QWidget *parent= 0, const char *name= 0);
+  KSnapshot(QWidget *parent= 0, const char *name= 0, bool grabCurrent=false);
   ~KSnapshot();
 
   enum CaptureMode { FullScreen=0, WindowUnderCursor=1, Region=2 };
@@ -98,6 +98,8 @@ private:
     void updatePreview();
     void performGrab();
     void autoincFilename();
+    int grabMode();
+    int timeout();
 
     QPixmap snapshot;
     QTimer grabTimer;
