@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: imlibwidget.h,v 1.11 2003/01/11 02:22:09 pfeiffer Exp $
+** $Id: imlibwidget.h,v 1.13 2004/12/29 02:58:09 pfeiffer Exp $
 **
 ** ImlibWidget: maps an Xlib window with Imlib's contents on a QWidget
 **
@@ -102,6 +102,8 @@ public:
   //  KuickImage *		find( const QString& filename );
 
 private:
+  ImlibImage *		loadImageWithQt( const QString& filename ) const;
+  
   int 			myMaxImages;
   QStringList		fileList;
   QPtrList<KuickImage>	kuickList;
@@ -171,6 +173,8 @@ public:
 
   ImlibData*	getImlibData() const 	       { return id; 		  }
 
+  virtual void  reparent( QWidget* parent, WFlags f, const QPoint& p, bool showIt = FALSE );
+
 public slots:
   void 		rotate90();
   void 		rotate270();
