@@ -16,6 +16,7 @@
     Boston, MA 02111-1307, USA.
 
 */
+// $Id: canvas.h,v 1.16 2005/01/26 15:06:14 mlaurent Exp $
 
 #ifndef KIMAGEVIEWER_CANVAS_H
 #define KIMAGEVIEWER_CANVAS_H
@@ -25,10 +26,11 @@ class QSize;
 class QImage;
 class QRect;
 class QPoint;
+class QWidget;
 
-#include <qobjectdefs.h>
 #include <qstring.h>
-
+#include <qobjectdefs.h>
+#include <kdemacros.h>
 namespace KImageViewer
 {
 
@@ -54,6 +56,7 @@ namespace KImageViewer
    }
    setCentralWidget( widget );
    connect( widget, SIGNAL( contextPress( const QPoint & ) ), SLOT( mySlot( const QPoint & ) ) );
+   connect( m_canvas->widget(), SIGNAL( contextPress( const QPoint & ) ), SLOT( mySlot( const QPoint & ) ) );
    </pre>
  *
  * You can't connect signals or slots using KImageViewer::Canvas, because this interface
@@ -64,7 +67,7 @@ namespace KImageViewer
  * until it's finished. So if you're using this interface please get in contact
  * with me.
  */
-class Canvas
+class KDE_EXPORT Canvas
 {
 	public:
 		/**
@@ -98,7 +101,9 @@ class Canvas
 		virtual double zoom() const = 0;
 
 		/**
-		 * returns the current (unzoomed) image
+		 * @return The current (unzoomed) image
+		 * Take care that the canas may delete the image so you probably need to
+		 * make a copy of the image if you want to keep it around.
 		 */
 		virtual const QImage * image() const = 0;
 
@@ -168,6 +173,11 @@ class Canvas
 		 */
 		virtual const QSize & minimumImageSize() const = 0;
 
+		/**
+		 * @return a pointer to the QWidget interface of this object
+		 */
+		virtual QWidget * widget() = 0;
+
 	signals:
 		/**
 		 * a mouse button was pressed and a context menu should be openend
@@ -214,6 +224,11 @@ class Canvas
 		 */
 		virtual void imageChanged() = 0;
 
+		/**
+		 * The current mouse cursor position on the image.
+		 */
+		virtual void cursorPos( const QPoint & ) = 0;
+
 	public slots:
 		/**
 		 * Set if the image should always be centered if the canvas is
@@ -222,12 +237,9 @@ class Canvas
 		virtual void setCentered( bool ) = 0;
 
 		/**
-		 * Give the canvas a new image to show.
-		 *
-		 * You may set the zoom for the new image from here. It defaults
-		 * to 100%
+		 * Give the canvas a new image to show. The zoom level is kept.
 		 */
-		virtual void setImage( const QImage &, double zoom = 1.0 ) = 0;
+		virtual void setImage( const QImage & ) = 0;
 
 		/**
 		 * Give the canvas a new image to show.
@@ -242,6 +254,11 @@ class Canvas
 		 */
 		virtual void setZoom( double ) = 0;
 
+		/**
+		 * Fit the image into the requested width and height.
+		 */
+		virtual void boundImageTo( const QSize & size ) = 0;
+
 		/**
 		 * Set the maximum size of the image. If this is set the image will
 		 * never exceed this size.
@@ -319,7 +336,8 @@ class Canvas
 		virtual void rotate( double angle, bool change = false ) = 0;
 		
 	protected:
-		Canvas() : m_iBlendEffect( 0 ) {}
+		Canvas();
+		virtual ~Canvas();
 		unsigned int m_iBlendEffect;
 
 }; //class Canvas
